"use strict";
import Antifurto from "./antifurto.js";
import Scenari from "./scenari.js";
import { luciToggleListener } from "./toggles.js";
import { makeElementBounce, showAlert } from "./alerts.js";
import Api from "./mqtt/api.js";
import Sensori from "./sensori.js";

const luceTemplate = {
    luogo: "",
    stato: false,
    id: "",
    in: "",
    out: "",
};

class Luci {
    /**
     * the div where everything related to the Luci is stored
     */
    static luciContainer = document.getElementById('luciContainer');
    /**
     * the array where all of the user's luci are stored
     */
    static luciBucket = [];
    /**
     * describes how many lights have already been added
     */
    static index = 0;

    constructor() {
        Luci.init();
        Luci.modalListener();
    }

    static init() {
        Luci.fillTable();

        const luciBtn = document.getElementById('mainButtonContainer').children[0];
        luciBtn.addEventListener('click', () => {
            Luci.toggleContainer(true);
            Scenari.toggleContainer(false);
            Antifurto.toggleContainer(false);
        }, false);

        // listen for the closing modal
        const cancelBtn = document.querySelector('#luci-modal .btn.btn-secondary');
        cancelBtn.addEventListener('click', () => {
            // delay 0.5s to allow the modal to close (otherwise it's not pretty)
            setTimeout(() => {
                Luci.selectReset();
            }, 500);
        }, false);
    }
    /**
     * shows and listens for the modal that allows the user to add a new light
     */
    static modalListener() {

        const selectStanza = document.getElementById('luogoLuce');
        const altroInput = document.getElementById('nuovaStanzaDIV');
        const altroClassList = altroInput.classList;
        const form = document.getElementById('search-form');
        const inputField = altroInput.querySelector('input');

        // show or hide the text input for the new room
        selectStanza.addEventListener('change', event => {
            event.preventDefault();
            const selected = event.target.value;

            if (selected === "altro") {
                altroClassList.remove('invisible');
                // set altroInput to required
                inputField.setAttribute('required', 'required');
            } else {
                altroClassList.add('invisible');
                // remove required from altroInput
                inputField.removeAttribute('required');
            }
        }, false);

        form.addEventListener('submit', event => {
            event.preventDefault();

            const selectedStanza = selectStanza.value;
            const selectIn = document.getElementById('scelta-porta-in');
            const selectedIn = selectIn.value;
            const selectOut = document.getElementById('scelta-porta-out');
            const selectedOut = selectOut.value;
            if (selectedStanza === "unselected" || selectedIn === "unselected" || selectedOut === "unselected") {
                alert("Selezionare i valori validi dai menu a tendina.");
                return;
            }

            let stanza = selectStanza.options[selectStanza.selectedIndex].text.trim(); // get selected option text
            if (selectedStanza === "altro") {
                const nuovaStanza = altroInput.querySelector('#nuovaStanza').value.trim();
                if (nuovaStanza == null || nuovaStanza == "" || nuovaStanza.toLowerCase() === "altro") {
                    alert("Inserire un nome valido per la stanza.");
                    return;
                }
                for (const stanza of Luci.luciBucket)
                    if (stanza.luogo === nuovaStanza) {
                        alert("Stanza già esistente");
                        return;
                    }
                stanza = nuovaStanza;
                // reset the select
                Luci.selectReset();
            }
            stanza = stanza[0].toUpperCase() + stanza.substring(1, stanza.length);
            // add stanza to table
            const luceJSON = Luci.createLight(stanza, false, selectedIn, selectedOut);
            Luci.sendLightToServer(luceJSON);

            // close modal
            const closeBtn = document.querySelector('#luci-modal .btn.btn-secondary');
            closeBtn.click();
        }, false);
    }
    /**
     * sends the light to the server and adds it to the table if the server responds with success
     * @param {luceTemplate} luce the light to be added to the table and the server
     */
    static async sendLightToServer(luce) {
        // HERE server send luce
        try {
            const apiRes = await Api.makeNewLuci(luce);
            if (apiRes == null) {
                showAlert(`${luce.luogo} creato`, "La nuova luce e' stata creata, usala subito!", true);
                Luci.showNewLight(luce);
            }

        } catch (error) {
            if (error.error)
                showAlert("Errore di comunicazione.", error.error, false);
        }
    }
    /**
     * creates a new light and adds it to the bucket
     * @param {String} luogo the name of the room
     * @param {Boolean} stato the status of the light
     * @returns {luceTemplate} the new light
     */
    static createLight(luogo, stato, valIn, valOut) {
        if(luogo == null)
            throw new Error("luogo non impostato");
        if (valIn == null || valOut == null || valIn.trim() == "" || valOut.trim() == "")
            throw new Error("i sensori in/out devono essere impostati");
        const luce = Object.assign({}, luceTemplate);
        luce.luogo = luogo[0].toUpperCase() + luogo.substring(1, luogo.length).toLowerCase();
        luce.stato = stato != null ? stato : false;
        luce.id = `luce-${Luci.index}`;
        luce.in = valIn.trim();
        luce.out = valOut.trim();
        Luci.luciBucket.push(luce);
        Luci.index++;
        return luce;
    }
    /**
     * shows a new light in the table after creating it
     * @param {luceTemplate} luceJSON the light to show
     */
    static showNewLight(luceJSON) {
        const selectIn = document.getElementById('scelta-porta-in');
        const sensoreIn = selectIn.value !== "unselected" ? selectIn.value : luceJSON.in;
        const selectOut = document.getElementById('scelta-porta-out');
        const sensoreOut = selectOut.value !== "unselected" ? selectOut.value : luceJSON.out;
        let sensoreInJSON = null;
        let sensoreOutJSON = null;
        try {
            if (selectIn.children.length > 1)
                sensoreInJSON = Sensori.removeSensoreOption(selectIn, sensoreIn/*,true,false,'Switch per luce in '+luceJSON.luogo*/);
            console.log(selectIn,sensoreInJSON)
            if (selectOut.children.length > 1)
                sensoreOutJSON = Sensori.removeSensoreOption(selectOut, sensoreOut/*,true,luceJSON.stato,'Luce situata in '+luceJSON.luogo*/);
            console.log(selectOut,sensoreOutJSON)
        } catch (error) {
            if (error.message === "Errore impossibile")
                showAlert("Errore", "Impossibile rimuovere il sensore dai menu a tendina.", false);
            else showAlert("Errore", error.message, false);
            Luci.resetBothSelectForSensors();
            Sensori.populateOptions(selectIn, false);
            Sensori.populateOptions(selectOut, true);
            return;
        }
        console.log(sensoreInJSON, sensoreOutJSON);
        if(sensoreInJSON?.utilizzo != null)
            sensoreInJSON.utilizzo = "Switch per luce in " + luceJSON.luogo;
        if(sensoreOutJSON?.utilizzo != null)
            sensoreOutJSON.utilizzo = "Luce situata in " + luceJSON.luogo;
        const luce = luceJSON.id == null ? Luci.createLight(luceJSON.luogo, luceJSON.stato, luceJSON.in, luceJSON.out) : luceJSON;
        const table = document.getElementById('table-row-luci');
        const row = document.createElement('tr');
        row.innerHTML = `
        <th scope="row" id="${luce.id}">${luce.luogo}</th>
        <td>
            <div class="switch-container no-box-sizing">
                <div class="toggle-button no-box-sizing ${luce.stato ? 'active' : ''}" data-bs-toggle="tooltip" data-bs-placement="bottom" title="${sensoreInJSON?.utilizzo}" data-sensore-in="${JSON.stringify(sensoreInJSON)}" data-sensore-out="${JSON.stringify(sensoreOutJSON)}">
                    <div class="inner-circle no-box-sizing" />
                </div>
            </div>
        </td>`;
        const lastRow = table.lastElementChild;
        table.insertBefore(row, lastRow);
        Sensori.initializaTooltips();
        // add listener for switch
        const toggle = row.querySelector('.toggle-button');
        luciToggleListener(toggle, luce);
        // remove options from the select
        const select = document.getElementById('luogoLuce');
        const options = select.options;
        const luogoLowercase = luce.luogo.toLowerCase();
        for (const option of options) {
            if (option.text.toLowerCase() === luogoLowercase) {
                select.removeChild(option);
                break;
            }
        }
    }
    /**
     * resets the select menu used to add a new light
     */
    static selectReset() {
        const select = document.getElementById('luogoLuce');
        const altroInput = document.getElementById('nuovaStanzaDIV');
        const inputField = altroInput.querySelector('input');

        select.selectedIndex = 0;
        altroInput.classList.add('invisible');
        inputField.removeAttribute('required');
        inputField.value = "";
    }
    /**
     * fills the table with the user's luci from the server
     */
    static async fillTable() {
        // HERE server get all lights
        try {
            const luci = await Api.getLuci(Luci.luciFromMqtt);
            // for each, add to table and to luciBucket
            if (await luci != null) {
                Luci.resetBothSelectForSensors();
                const selectIn = document.getElementById('scelta-porta-in');
                Sensori.populateOptions(selectIn, false);
                const selectOut = document.getElementById('scelta-porta-out');
                Sensori.populateOptions(selectOut, true);
                for (const luce of luci)
                    Luci.showNewLight(luce);
            }
        } catch (error) {
            if (error.error)
                showAlert("Errore di comunicazione", error.error, false);
            else if (error.message)
                showAlert("Errore", error.message, false);
        }
    }

    static luciFromMqtt(luci){
        Luci.resetBothSelectForSensors();
        const selectIn = document.getElementById('scelta-porta-in');
        Sensori.populateOptions(selectIn, false);
        const selectOut = document.getElementById('scelta-porta-out');
        Sensori.populateOptions(selectOut, true);
        for (const luce of luci)
            Luci.showNewLight(luce);
    }

    staticUpdateStatusAfterMqtt(luci){
        console.log(luci)
        for (const luce of luci) {
            const table = document.getElementById('table-row-luci');
            const luceJSON = Luci.luciBucket.find(old_luce => old_luce.out === luce.out);
            const th = table.querySelector(`#${luceJSON.id}`);
            const td = th.nextElementSibling;
            if(luce.stato !== luceJSON.stato){
                // click the toggle
                const toggle = td.querySelector('.toggle-button');
                toggle.click();
            }
        }
    }
    /**
     * toggle the visibility of the luci container
     * @param {Boolean} visible 
     */
    static toggleContainer(visible) {
        if (visible) {
            Luci.luciContainer.classList.remove('invisible');
            if (Luci.luciBucket.length === 0) {
                // make the + button bounce
                const plusBtn = document.querySelector('.fa-circle-plus');
                makeElementBounce(plusBtn);
            }
        } else {
            Luci.luciContainer.classList.add('invisible');
        }
    }

    static resetBothSelectForSensors() {
        const selectIn = document.getElementById('scelta-porta-in');
        const selectOut = document.getElementById('scelta-porta-out');
        // only have <option value="unselected">Scegli una porta di attivazione</option> in both selects
        // remove any other node
        const firstChildIn = selectIn.firstElementChild;
        const firstChildOut = selectOut.firstElementChild;
        while (selectIn.lastElementChild !== firstChildIn) {
            selectIn.removeChild(selectIn.lastElementChild);
        }
        while (selectOut.lastElementChild !== firstChildOut) {
            selectOut.removeChild(selectOut.lastElementChild);
        }
    }
}

export default Luci;